@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 2005 - 2006 The GNOME Foundation.
+ * Copyright (C) 2005 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -49,9 +49,18 @@ struct _GdaDataModelQueryPrivate {
 	GdaDataModel     *data;
 	GError           *refresh_error; /* if @data is NULL, then can contain the error */
 
-	gboolean          defer_refresh;
-	gboolean          refresh_pending;
+	gboolean          multiple_updates;
+	gboolean          needs_refresh;
 	GSList           *columns;
+
+	gboolean          transaction_allowed;
+	gboolean          transaction_started; /* true if we have started a transaction here */
+	gboolean          transaction_needs_commit;
+	guint             svp_id;   /* counter for savepoints */
+	gchar            *svp_name; /* non NULL if we have created a savepoint here */
+	/* note: transaction_started and svp_name are mutually exclusive */
+
+	gboolean          emit_reset;
 };
 
 /* properties */
@@ -61,7 +70,8 @@ enum
         PROP_SEL_QUERY,
 	PROP_INS_QUERY,
 	PROP_UPD_QUERY,
-	PROP_DEL_QUERY
+	PROP_DEL_QUERY,
+	PROP_USE_TRANSACTION
 };
 
 enum 
@@ -92,9 +102,9 @@ static void                 gda_data_model_query_data_model_init (GdaDataModelCl
 static gint                 gda_data_model_query_get_n_rows      (GdaDataModel *model);
 static gint                 gda_data_model_query_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_query_describe_column (GdaDataModel *model, gint col);
-static guint                gda_data_model_query_get_access_flags(GdaDataModel *model);
+static GdaDataModelAccessFlags gda_data_model_query_get_access_flags(GdaDataModel *model);
 static const GValue        *gda_data_model_query_get_value_at    (GdaDataModel *model, gint col, gint row);
-static guint                gda_data_model_query_get_attributes_at (GdaDataModel *model, gint col, gint row);
+static GdaValueAttribute    gda_data_model_query_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
 static GdaDataModelIter    *gda_data_model_query_create_iter     (GdaDataModel *model);
 
@@ -113,6 +123,9 @@ static void create_columns (GdaDataModelQuery *model);
 static void to_be_destroyed_query_cb (GdaQuery *query, GdaDataModelQuery *model);
 static void param_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GdaDataModelQuery *model);
 
+static void opt_start_transaction_or_svp (GdaDataModelQuery *selmodel);
+static void opt_end_transaction_or_svp (GdaDataModelQuery *selmodel);
+
 #ifdef GDA_DEBUG
 static void gda_data_model_query_dump (GdaDataModelQuery *select, guint offset);
 #endif
@@ -139,7 +152,7 @@ gda_data_model_query_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelQueryClass),
 			(GBaseInitFunc) NULL,
@@ -199,6 +212,12 @@ gda_data_model_query_class_init (GdaDataModelQueryClass *klass)
 							       "DELETE Query to be executed to remove data",
                                                                GDA_TYPE_QUERY,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE));
+	
+	g_object_class_install_property (object_class, PROP_USE_TRANSACTION,
+                                         g_param_spec_boolean ("use_transaction", "Use transaction", 
+							       "Run modification queries within a transaction",
+                                                               FALSE,
+							       G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 
 	/* virtual functions */
 	object_class->dispose = gda_data_model_query_dispose;
@@ -248,8 +267,16 @@ gda_data_model_query_init (GdaDataModelQuery *model, GdaDataModelQueryClass *kla
 	model->priv->columns = NULL;
 
 	/* model refreshing is performed as soon as any modification is done */
-	model->priv->defer_refresh = FALSE;
-	model->priv->refresh_pending = FALSE;
+	model->priv->multiple_updates = FALSE;
+	model->priv->needs_refresh = FALSE;
+
+	model->priv->transaction_allowed = FALSE;
+	model->priv->transaction_started = FALSE;
+	model->priv->transaction_needs_commit = FALSE;
+	model->priv->svp_id = 0;
+	model->priv->svp_name = NULL;
+
+	model->priv->emit_reset = FALSE;
 }
 
 static void
@@ -262,6 +289,10 @@ gda_data_model_query_dispose (GObject *object)
 	/* free memory */
 	if (model->priv) {
 		gint i;
+
+		if (model->priv->transaction_started || model->priv->svp_name)
+			opt_end_transaction_or_svp (model);
+
 		if (model->priv->columns) {
 			g_slist_foreach (model->priv->columns, (GFunc) g_object_unref, NULL);
 			g_slist_free (model->priv->columns);
@@ -350,8 +381,9 @@ param_name_to_int (const gchar *pname, gint *result, gboolean *old_val)
 static void
 check_param_type (GdaParameter *param, GType type)
 {
-	if (type !=  gda_parameter_get_g_type (param)) {
-		g_warning (_("Type of parameter '%s' is '%s' when it should be '%s', GdaDataModelQuery object will now work correctly"),
+	if ((type != 0) && (type !=  gda_parameter_get_g_type (param))) {
+		g_warning (_("Type of parameter '%s' is '%s' when it should be '%s', "
+			     "GdaDataModelQuery object will not work correctly"),
 			   gda_object_get_name (GDA_OBJECT (param)), 
 			   g_type_name (gda_parameter_get_g_type (param)),
 			   g_type_name (type));
@@ -396,12 +428,9 @@ gda_data_model_query_set_property (GObject *object,
 					GSList *targets, *tlist;
 
 					targets = gda_query_get_targets (model->priv->queries[qindex]);
-					tlist = targets;
-					while (tlist) {
+					for (tlist = targets; tlist; tlist = tlist->next)
 						g_slist_free (gda_query_expand_all_field (model->priv->queries[qindex],
 											  GDA_QUERY_TARGET (tlist->data)));
-						tlist = g_slist_next (tlist);
-					}
 					g_slist_free (targets);
 
 					g_signal_connect (model->priv->queries[qindex], "changed",
@@ -446,6 +475,8 @@ gda_data_model_query_set_property (GObject *object,
 									    gda_column_get_default_value (col))
 										gda_parameter_set_exists_default_value
 										 ((GdaParameter *)(params->data), TRUE);
+									gda_object_set_id (GDA_OBJECT (param),
+											   gda_column_get_name (col));
 								}
 							}
 							else {
@@ -474,6 +505,9 @@ gda_data_model_query_set_property (GObject *object,
 				}
 			}
 			break;
+		case PROP_USE_TRANSACTION:
+			model->priv->transaction_allowed = g_value_get_boolean (value);
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -499,6 +533,9 @@ gda_data_model_query_get_property (GObject *object,
 		case PROP_DEL_QUERY:
 			g_value_set_object (value, G_OBJECT (model->priv->queries[qindex]));
 			break;
+		case PROP_USE_TRANSACTION:
+			g_value_set_boolean (value, model->priv->transaction_allowed);
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -556,7 +593,7 @@ param_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GdaDataModel
 			g_object_unref (model->priv->data);
 			model->priv->data = NULL;
 		}
-		gda_data_model_signal_emit_changed ((GdaDataModel *) model);
+		gda_data_model_reset ((GdaDataModel *) model);
 	}
 }
 
@@ -647,7 +684,7 @@ gda_data_model_query_refresh (GdaDataModelQuery *model, GError **error)
 		return FALSE;
 	}
 
-	gda_data_model_signal_emit_changed ((GdaDataModel *) model);
+	gda_data_model_reset ((GdaDataModel *) model);
 	return model->priv->data ? TRUE : FALSE;
 }
 
@@ -776,25 +813,25 @@ gda_data_model_query_get_n_columns (GdaDataModel *model)
 static void
 create_columns (GdaDataModelQuery *model)
 {
+	GSList *fields;
+
 	if (model->priv->columns)
 		return;
 	if (! model->priv->queries[SEL_QUERY])
 		return;
 
-	gda_referer_activate (GDA_REFERER (model->priv->queries[SEL_QUERY]));	
-	if (gda_referer_is_active (GDA_REFERER (model->priv->queries[SEL_QUERY]))) {
-		GSList *fields, *list;
+	gda_referer_activate (GDA_REFERER (model->priv->queries[SEL_QUERY]));
+	fields = gda_entity_get_fields (GDA_ENTITY (model->priv->queries[SEL_QUERY]));
+	if (gda_referer_is_active (GDA_REFERER (model->priv->queries[SEL_QUERY])) && fields) {
+		GSList *list;
 		gboolean allok = TRUE;
-
-		fields = gda_entity_get_fields (GDA_ENTITY (model->priv->queries[SEL_QUERY]));
-		list = fields;
-		while (list && allok) {
+		
+		for (list = fields; list && allok; list = list->next) {
 			if (gda_entity_field_get_g_type (GDA_ENTITY_FIELD (list->data)) == G_TYPE_INVALID) {
 				allok = FALSE;
 				g_warning (_("Can't determine the GType for field '%s', please fill a bug report"), 
 					   gda_object_get_name (GDA_OBJECT (list->data)));
 			}
-			list = g_slist_next (list);
 		}
 		if (! allok) 
 			return;
@@ -847,10 +884,17 @@ create_columns (GdaDataModelQuery *model)
 				model->priv->columns = g_slist_append (model->priv->columns, col);
 			}
 		}
-		else
-			g_warning (_("Can't compute the list of columns because the "
-				   "SELECT query is not active (you can use a dictionary to correct this) "
-				   "and the model does not contain any data"));
+		else {
+			/* Can't compute the list of columns because the SELECT query is not active 
+			 * and the model does not contain any data;
+			 * next time we emit a "reset" signal */
+			model->priv->emit_reset = TRUE;
+		}
+	}
+
+	if (model->priv->columns && model->priv->emit_reset) {
+		model->priv->emit_reset = FALSE;
+		gda_data_model_reset (GDA_DATA_MODEL (model));
 	}
 }
 
@@ -872,11 +916,11 @@ gda_data_model_query_describe_column (GdaDataModel *model, gint col)
 		return NULL;
 }
 
-static guint
+static GdaDataModelAccessFlags
 gda_data_model_query_get_access_flags (GdaDataModel *model)
 {
 	GdaDataModelQuery *selmodel;
-	guint flags = GDA_DATA_MODEL_ACCESS_RANDOM;
+	GdaDataModelAccessFlags flags = GDA_DATA_MODEL_ACCESS_RANDOM;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_QUERY (model), 0);
 	selmodel = GDA_DATA_MODEL_QUERY (model);
@@ -954,10 +998,10 @@ gda_data_model_query_get_value_at (GdaDataModel *model, gint col, gint row)
 		return NULL;
 }
 
-static guint
+static GdaValueAttribute
 gda_data_model_query_get_attributes_at (GdaDataModel *model, gint col, gint row)
 {
-	guint flags = 0;
+	GdaValueAttribute flags = 0;
 	GdaDataModelQuery *selmodel;
 	GdaParameter *p_used = NULL;
 
@@ -1038,26 +1082,88 @@ gda_data_model_query_create_iter (GdaDataModel *model)
 	return iter;
 }
 
-static gboolean
-run_modif_query (GdaDataModelQuery *selmodel, gint query_type, GError **error)
+static gchar *try_add_savepoint (GdaDataModelQuery *selmodel);
+static void   try_remove_savepoint (GdaDataModelQuery *selmodel, const gchar *svp_name);
+
+static gchar *
+try_add_savepoint (GdaDataModelQuery *selmodel)
 {
-	gboolean retval = FALSE;
+	GdaConnection *cnc;
+	gchar *svp_name = NULL;
+	cnc = gda_dict_get_connection (gda_object_get_dict (GDA_OBJECT (selmodel->priv->queries[SEL_QUERY])));
+	if (cnc && gda_connection_supports_feature (cnc, GDA_CONNECTION_FEATURE_SAVEPOINTS)) {
+		gchar *name;
+		
+		name = g_strdup_printf ("_gda_data_model_query_svp_%p_%d", selmodel, selmodel->priv->svp_id++);
+		if (gda_connection_add_savepoint (cnc, name, NULL))
+			svp_name = name;
+		else
+			g_free (name);
+	}
+	return svp_name;
+}
+
+static void
+try_remove_savepoint (GdaDataModelQuery *selmodel, const gchar *svp_name)
+{
+	GdaConnection *cnc;
+
+	if (!svp_name)
+		return;
+	cnc = gda_dict_get_connection (gda_object_get_dict (GDA_OBJECT (selmodel->priv->queries[SEL_QUERY])));
+	if (cnc && gda_connection_supports_feature (cnc, GDA_CONNECTION_FEATURE_SAVEPOINTS_REMOVE))
+		gda_connection_delete_savepoint (cnc, svp_name, NULL);
+}
+
+static gboolean
+run_modify_query (GdaDataModelQuery *selmodel, gint query_type, GError **error)
+{
+	gboolean modify_done = FALSE;
 	GdaParameterList *plist;
+	gchar *svp_name = NULL;
+
+	/* try to add a savepoint if we are not doing multiple updates */
+	if (!selmodel->priv->multiple_updates)
+		svp_name = try_add_savepoint (selmodel);
+
 	plist = (GdaParameterList *) gda_query_execute (selmodel->priv->queries[query_type], 
 							selmodel->priv->params [query_type], TRUE, error);
 	if (plist) {
-		retval = TRUE;
+		modify_done = TRUE;
 		g_object_unref (plist);
 	}
 
-	if (retval && !selmodel->priv->defer_refresh)
-		/* do a refresh */
+	if (modify_done) {
+		/* modif query executed without error */
+		if (!selmodel->priv->multiple_updates)
+			gda_data_model_query_refresh (selmodel, NULL);
+		else 
+			selmodel->priv->needs_refresh = TRUE;
+	}
+	else {
+		/* modif query did not execute correctly */
+		if (!selmodel->priv->multiple_updates)
+			/* nothing to do */;
+		else
+			selmodel->priv->transaction_needs_commit = FALSE;
+	}
+
+	/* try to remove the savepoint added above */
+	if (svp_name) {
+		try_remove_savepoint (selmodel, svp_name);
+		g_free (svp_name);
+	}
+
+#ifdef REMOVE
+	if (modify_done && !selmodel->priv->multiple_updates)
+		/* modifications were successfull => refresh needed */
 		gda_data_model_query_refresh (selmodel, NULL);
 	else
 		/* the refresh is delayed */
-		selmodel->priv->refresh_pending = TRUE;
+		selmodel->priv->needs_refresh = TRUE;
+#endif
 
-	return retval;
+	return modify_done;
 }
 
 static gboolean
@@ -1105,7 +1211,7 @@ gda_data_model_query_set_value_at (GdaDataModel *model, gint col, gint row, cons
 	}
 
 	/* render the SQL and run it */
-	return run_modif_query (selmodel, UPD_QUERY, error);
+	return run_modify_query (selmodel, UPD_QUERY, error);
 }
 
 static gboolean
@@ -1155,7 +1261,7 @@ gda_data_model_query_set_values (GdaDataModel *model, gint row, GList *values, G
 	}
 
 	/* render the SQL and run it */
-	return run_modif_query (selmodel, UPD_QUERY, error);
+	return run_modify_query (selmodel, UPD_QUERY, error);
 }
 
 static gint
@@ -1198,7 +1304,7 @@ gda_data_model_query_append_values (GdaDataModel *model, const GList *values, GE
 	}
 
 	/* render the SQL and run it */
-	retval = run_modif_query (selmodel, INS_QUERY, error);
+	retval = run_modify_query (selmodel, INS_QUERY, error);
 
 	if (retval)
 		return 0;
@@ -1241,7 +1347,7 @@ gda_data_model_query_append_row (GdaDataModel *model, GError **error)
 	}
 
 	/* render the SQL and run it */
-	retval = run_modif_query (selmodel, INS_QUERY, error);
+	retval = run_modify_query (selmodel, INS_QUERY, error);
 
 	if (retval)
 		return 0;
@@ -1283,9 +1389,99 @@ gda_data_model_query_remove_row (GdaDataModel *model, gint row, GError **error)
 	}
 
 	/* render the SQL and run it */
-	return run_modif_query (selmodel, DEL_QUERY, error);
+	return run_modify_query (selmodel, DEL_QUERY, error);
 }
 
+static void
+opt_start_transaction_or_svp (GdaDataModelQuery *selmodel)
+{
+	GdaConnection *cnc;
+
+	/* if the data model is not allowed to start transactions, then nothing to do */
+	if (!selmodel->priv->transaction_allowed)
+		return;
+
+	/* if we already have started a transaction, or added a savepoint, then nothing to do */
+	if (selmodel->priv->transaction_started ||
+	    selmodel->priv->svp_name)
+		return;
+
+	/* re-init internal status */
+	selmodel->priv->transaction_needs_commit = FALSE;
+
+	/* start a transaction is no transaction has yet been started */
+	cnc = gda_dict_get_connection (gda_object_get_dict (GDA_OBJECT (selmodel->priv->queries[SEL_QUERY])));
+	if (cnc && gda_connection_supports_feature (cnc, GDA_CONNECTION_FEATURE_TRANSACTIONS)) {
+		GdaTransactionStatus *tstat;
+		
+		tstat = gda_connection_get_transaction_status (cnc);
+		if (tstat) {
+			/* a transaction already exists, try to use a savepoint */
+			if (gda_connection_supports_feature (cnc, GDA_CONNECTION_FEATURE_SAVEPOINTS)) {
+				gchar *name;
+				
+				name = g_strdup_printf ("_gda_data_model_query_svp_%p_%d", selmodel, selmodel->priv->svp_id++);
+				if (gda_connection_add_savepoint (cnc, name, NULL))
+					selmodel->priv->svp_name = name;
+				else
+					g_free (name);
+			}
+		}
+		else 
+			/* start a transaction */
+			selmodel->priv->transaction_started = gda_connection_begin_transaction (cnc, NULL,
+							      GDA_TRANSACTION_ISOLATION_UNKNOWN, NULL);
+	}
+
+	/* init internal status */
+	selmodel->priv->transaction_needs_commit = TRUE;
+
+	if (selmodel->priv->transaction_started)
+		g_print ("GdaDataModelQuery %p: started transaction\n", selmodel);
+	if (selmodel->priv->svp_name)
+		g_print ("GdaDataModelQuery %p: added savepoint %s\n", selmodel, selmodel->priv->svp_name);
+}
+
+static void
+opt_end_transaction_or_svp (GdaDataModelQuery *selmodel)
+{
+	GdaConnection *cnc;
+
+	if (!selmodel->priv->transaction_started && !selmodel->priv->svp_name)
+		return;
+
+	g_print ("GdaDataModelQuery %p (END1): %s\n", selmodel, selmodel->priv->needs_refresh ? "needs refresh" : "no refresh");
+	cnc = gda_dict_get_connection (gda_object_get_dict (GDA_OBJECT (selmodel->priv->queries[SEL_QUERY])));
+	if (selmodel->priv->transaction_started) {
+		g_assert (!selmodel->priv->svp_name);
+		if (selmodel->priv->transaction_needs_commit) {
+			/* try to commit transaction */
+			if (!gda_connection_commit_transaction (cnc, NULL, NULL))
+				selmodel->priv->needs_refresh = TRUE;
+		}
+		else {
+			/* rollback transaction */
+			selmodel->priv->needs_refresh = gda_connection_rollback_transaction (cnc, NULL, NULL) ? FALSE : TRUE;
+		}
+		selmodel->priv->transaction_started = FALSE;
+	}
+	else {
+		if (!selmodel->priv->transaction_needs_commit) {
+			selmodel->priv->needs_refresh = gda_connection_rollback_savepoint (cnc, selmodel->priv->svp_name, NULL) ?
+				FALSE : TRUE;
+		}
+		else
+			if (gda_connection_supports_feature (cnc, GDA_CONNECTION_FEATURE_SAVEPOINTS_REMOVE))
+				if (!gda_connection_delete_savepoint (cnc, selmodel->priv->svp_name, NULL))
+					selmodel->priv->needs_refresh = TRUE;
+		g_free (selmodel->priv->svp_name);
+		selmodel->priv->svp_name = NULL;
+	}
+	g_print ("GdaDataModelQuery %p (END2): %s\n", selmodel, selmodel->priv->needs_refresh ? "needs refresh" : "no refresh");
+}
+
+
+
 static void
 gda_data_model_query_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GValue *hint_value)
 {
@@ -1297,12 +1493,16 @@ gda_data_model_query_send_hint (GdaDataModel *model, GdaDataModelHint hint, cons
 	if (hint == GDA_DATA_MODEL_HINT_REFRESH)
 		gda_data_model_query_refresh (selmodel, NULL);
 	else {
-		if (hint == GDA_DATA_MODEL_HINT_START_BATCH_UPDATE)
-			selmodel->priv->defer_refresh = TRUE;
+		if (hint == GDA_DATA_MODEL_HINT_START_BATCH_UPDATE) {
+			opt_start_transaction_or_svp (selmodel);
+			selmodel->priv->multiple_updates = TRUE;
+		}
 		else {
 			if (hint == GDA_DATA_MODEL_HINT_END_BATCH_UPDATE) {
-				selmodel->priv->defer_refresh = FALSE;
-				if (selmodel->priv->refresh_pending)
+				selmodel->priv->multiple_updates = FALSE;
+				opt_end_transaction_or_svp (selmodel);
+					
+				if (selmodel->priv->needs_refresh)
 					gda_data_model_query_refresh (selmodel, NULL);
 			}
 		}
@@ -1313,12 +1513,12 @@ gda_data_model_query_send_hint (GdaDataModel *model, GdaDataModelHint hint, cons
  * Computation of INSERT, DELETE and UPDATE queries
  */
 
-static GdaQueryTarget *auto_compute_assert_modif_target (GdaDataModelQuery *model, const gchar *target, GError **error);
-static GSList   *auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modif_target, 
+static GdaQueryTarget *auto_compute_assert_modify_target (GdaDataModelQuery *model, const gchar *target, GError **error);
+static GSList   *auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modify_target, 
 						      gboolean use_all_if_no_pk, GError **error);
-static GSList   *auto_compute_make_mod_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modif_target,
+static GSList   *auto_compute_make_mod_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modify_target,
 						     GError **error);
-static void      auto_compute_add_mod_fields_to_query (GdaDataModelQuery *model, GdaQueryTarget *modif_target,
+static void      auto_compute_add_mod_fields_to_query (GdaDataModelQuery *model, GdaQueryTarget *modify_target,
 						       GSList *mod_query_fields, GdaQuery *query);
 static void      auto_compute_add_where_cond_to_query (GdaDataModelQuery *model, GSList *mod_query_fields, 
 						       GdaQuery *query);
@@ -1346,7 +1546,7 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 	GSList *cond_query_fields; /* list of GdaQueryField objects to be used in WHERE condition */
 	GSList *mod_query_fields;  /* list of GdaQueryField objects to be modified in INSERT and UPDATE */
 	GdaQuery *query;
-	GdaQueryTarget *modif_target;
+	GdaQueryTarget *modify_target;
 
 	if (!model->priv->queries[SEL_QUERY]) {
 		g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR,
@@ -1362,18 +1562,18 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 		return FALSE;
 	}
 
-	modif_target = auto_compute_assert_modif_target (model, target, error);
-	if (! modif_target)
+	modify_target = auto_compute_assert_modify_target (model, target, error);
+	if (! modify_target)
 		return FALSE;
 
 	/* make lists of fields used to create INSERT, DELETE and UPDATE queries */
-	cond_query_fields = auto_compute_make_cond_query_fields (model, modif_target, 
+	cond_query_fields = auto_compute_make_cond_query_fields (model, modify_target, 
 								 options & GDA_DATA_MODEL_QUERY_OPTION_USE_ALL_FIELDS_IF_NO_PK,
 								 error);
 	if (!cond_query_fields)
 		return FALSE;
 
-	mod_query_fields = auto_compute_make_mod_query_fields (model, modif_target, error);
+	mod_query_fields = auto_compute_make_mod_query_fields (model, modify_target, error);
 	if (!mod_query_fields) {
 		g_slist_free (cond_query_fields);
 		return FALSE;
@@ -1382,7 +1582,7 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 	/* compute UPDATE query */
 	query = gda_query_new (gda_object_get_dict (GDA_OBJECT (model->priv->queries[SEL_QUERY])));
 	gda_query_set_query_type (query, GDA_QUERY_TYPE_UPDATE);
-	auto_compute_add_mod_fields_to_query (model, modif_target, mod_query_fields, query);
+	auto_compute_add_mod_fields_to_query (model, modify_target, mod_query_fields, query);
 	auto_compute_add_where_cond_to_query (model, cond_query_fields, query);
 	g_object_set (G_OBJECT (model), "update_query", query, NULL);
 	g_object_unref (query);
@@ -1390,7 +1590,7 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 	/* compute INSERT query */
 	query = gda_query_new (gda_object_get_dict (GDA_OBJECT (model->priv->queries[SEL_QUERY])));
 	gda_query_set_query_type (query, GDA_QUERY_TYPE_INSERT);
-	auto_compute_add_mod_fields_to_query (model, modif_target, mod_query_fields, query);
+	auto_compute_add_mod_fields_to_query (model, modify_target, mod_query_fields, query);
 	g_object_set (G_OBJECT (model), "insert_query", query, NULL);
 	g_object_unref (query);
 
@@ -1401,7 +1601,7 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 		GdaQueryTarget *target;
 		GdaDictTable *mod_table;
 		
-		mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modif_target));
+		mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modify_target));
 		target = gda_query_target_new (query, gda_object_get_name (GDA_OBJECT (mod_table)));
 		gda_query_add_target (query, target, NULL);
 		g_object_unref (target);
@@ -1417,16 +1617,16 @@ gda_data_model_query_compute_modification_queries (GdaDataModelQuery *model, con
 }
 
 static GdaQueryTarget *
-auto_compute_assert_modif_target (GdaDataModelQuery *model, const gchar *target, GError **error)
+auto_compute_assert_modify_target (GdaDataModelQuery *model, const gchar *target, GError **error)
 {
 	/* 
-	 * ensure that model->priv->modif_target is not NULL and represents a dict table 
+	 * ensure that model->priv->modify_target is not NULL and represents a dict table 
 	 */
-	GdaQueryTarget *modif_target;
+	GdaQueryTarget *modify_target;
 
 	if (target && *target) {
-		modif_target = gda_query_get_target_by_alias (model->priv->queries[SEL_QUERY], target);
-		if (!modif_target) {
+		modify_target = gda_query_get_target_by_alias (model->priv->queries[SEL_QUERY], target);
+		if (!modify_target) {
 			g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR,
 				     GDA_DATA_MODEL_QUERY_COMPUTE_MODIF_QUERIES_ERROR,
 				     _("Could not identify target table '%s' to modify in SELECT query"), target);
@@ -1435,7 +1635,7 @@ auto_compute_assert_modif_target (GdaDataModelQuery *model, const gchar *target,
 		else {
 			GdaEntity *ent;
 			
-			ent = gda_query_target_get_represented_entity (modif_target);
+			ent = gda_query_target_get_represented_entity (modify_target);
 			if (!GDA_IS_DICT_TABLE (ent)) {
 				g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR,
 					     GDA_DATA_MODEL_QUERY_COMPUTE_MODIF_QUERIES_ERROR,
@@ -1466,22 +1666,22 @@ auto_compute_assert_modif_target (GdaDataModelQuery *model, const gchar *target,
 		if (table_targets->next) {
 			g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR,
 				     GDA_DATA_MODEL_QUERY_COMPUTE_MODIF_QUERIES_ERROR,
-				     _("Ambiguous target table to modify in SELECT query"));
+				     _("More than one target table can be modified in SELECT query, specify which one"));
 			g_slist_free (table_targets);
 			g_slist_free (targets);
 			return FALSE;
 		}
 		
-		modif_target = GDA_QUERY_TARGET (table_targets->data);
+		modify_target = GDA_QUERY_TARGET (table_targets->data);
 		g_slist_free (table_targets);
 		g_slist_free (targets);
 	}
 	
-	return modif_target;
+	return modify_target;
 }
 
 static GSList *
-auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modif_target, 
+auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modify_target, 
 				     gboolean use_all_if_no_pk, GError **error)
 {
 	/*
@@ -1492,7 +1692,7 @@ auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *m
 	GSList *cond_query_fields = NULL;
 	gboolean error_is_set = FALSE;
 		
-	mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modif_target));
+	mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modify_target));
 	pk_constraint = gda_dict_table_get_pk_constraint (mod_table);
 	if (pk_constraint) {
 		/* use fields is Pk */
@@ -1502,7 +1702,7 @@ auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *m
 		gboolean found;
 		cond_dict_fields = gda_dict_constraint_pkey_get_fields (pk_constraint);
 		target_fields = gda_query_get_fields_by_target (model->priv->queries[SEL_QUERY],
-								modif_target, TRUE);
+								modify_target, TRUE);
 
 		/* convert list of GdaQueryField objects (target_fields) 
 		 * to list of GdaDictFieldField objects (query_fields) */
@@ -1546,7 +1746,11 @@ auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *m
 		if (use_all_if_no_pk) {
 			/* use ALL the present fields to make a WHERE condition */
 			cond_query_fields = gda_query_get_fields_by_target (model->priv->queries[SEL_QUERY],
-									    modif_target, TRUE);
+									    modify_target, TRUE);
+			if (error_is_set && error) {
+				g_error_free (*error);
+				*error = NULL;
+			}
 		}
 		else {
 			if (!error_is_set) {
@@ -1562,16 +1766,16 @@ auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *m
 }
 
 static GSList *
-auto_compute_make_mod_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modif_target, GError **error)
+auto_compute_make_mod_query_fields (GdaDataModelQuery *model, GdaQueryTarget *modify_target, GError **error)
 {
 	GSList *mod_query_fields = NULL;
 	GSList *mod_dict_fields = NULL;
 	GSList *target_fields, *list;
 	gboolean duplicate = FALSE;
 
-	/* make sure there are no duplicates in the fields referenced by modif_target */
+	/* make sure there are no duplicates in the fields referenced by modify_target */
 	target_fields = gda_query_get_fields_by_target (model->priv->queries[SEL_QUERY],
-							modif_target, TRUE);
+							modify_target, TRUE);
 	for (list = target_fields; list && !duplicate; list = list->next) {
 		GdaQueryFieldField *qfield = (GdaQueryFieldField *) (list->data);
 		GdaEntityField *efield;
@@ -1587,7 +1791,7 @@ auto_compute_make_mod_query_fields (GdaDataModelQuery *model, GdaQueryTarget *mo
 			GdaDictTable *mod_table;
 			duplicate = TRUE;
 
-			mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modif_target));
+			mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modify_target));
 			g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR,
 				     GDA_DATA_MODEL_QUERY_COMPUTE_MODIF_QUERIES_ERROR,
 				     _("Field '%s.%s' appears more than once in SELECT query"),
@@ -1608,14 +1812,14 @@ auto_compute_make_mod_query_fields (GdaDataModelQuery *model, GdaQueryTarget *mo
 }
 
 static void
-auto_compute_add_mod_fields_to_query (GdaDataModelQuery *model, GdaQueryTarget *modif_target, 
+auto_compute_add_mod_fields_to_query (GdaDataModelQuery *model, GdaQueryTarget *modify_target, 
 				      GSList *mod_query_fields, GdaQuery *query)
 {
 	GdaQueryTarget *target;
 	GdaDictTable *mod_table;
 	GSList *list;
 
-	mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modif_target));
+	mod_table = GDA_DICT_TABLE (gda_query_target_get_represented_entity (modify_target));
 	target = gda_query_target_new (query, gda_object_get_name (GDA_OBJECT (mod_table)));
 	gda_query_add_target (query, target, NULL);
 	g_object_unref (target);
