@@ -1,21 +1,21 @@
 /* gda-query-field-agg.c
  *
- * Copyright (C) 2005 - 2006 Vivien Malerba
+ * Copyright (C) 2005 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -58,11 +58,12 @@ static gboolean     gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xml
 /* Field interface */
 static void         gda_query_field_agg_field_init      (GdaEntityFieldIface *iface);
 static GdaEntity   *gda_query_field_agg_get_entity      (GdaEntityField *iface);
-static GdaDictType *gda_query_field_agg_get_data_type   (GdaEntityField *iface);
+static GdaDictType *gda_query_field_agg_get_dict_type   (GdaEntityField *iface);
 
 /* Renderer interface */
 static void         gda_query_field_agg_renderer_init   (GdaRendererIface *iface);
-static gchar       *gda_query_field_agg_render_as_sql   (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error);
+static gchar       *gda_query_field_agg_render_as_sql   (GdaRenderer *iface, GdaParameterList *context, 
+							 GSList **out_params_used, GdaRendererOptions options, GError **error);
 static gchar       *gda_query_field_agg_render_as_str   (GdaRenderer *iface, GdaParameterList *context);
 
 /* Referer interface */
@@ -123,7 +124,7 @@ gda_query_field_agg_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaQueryFieldAggClass),
 			(GBaseInitFunc) NULL,
@@ -182,7 +183,7 @@ static void
 gda_query_field_agg_field_init (GdaEntityFieldIface *iface)
 {
 	iface->get_entity = gda_query_field_agg_get_entity;
-	iface->get_data_type = gda_query_field_agg_get_data_type;
+	iface->get_dict_type = gda_query_field_agg_get_dict_type;
 }
 
 static void
@@ -594,7 +595,7 @@ gda_query_field_agg_set_arg (GdaQueryFieldAgg *agg, GdaQueryField *arg)
  * Returns: the #GnomeDbQField argument
  */
 GdaQueryField *
-gda_query_field_agg_get_args (GdaQueryFieldAgg *agg)
+gda_query_field_agg_get_arg (GdaQueryFieldAgg *agg)
 {
 	g_return_val_if_fail (agg && GDA_IS_QUERY_FIELD_AGG (agg), NULL);
 	g_return_val_if_fail (agg->priv, NULL);
@@ -652,7 +653,7 @@ gda_query_field_agg_get_entity (GdaEntityField *iface)
 }
 
 static GdaDictType *
-gda_query_field_agg_get_data_type (GdaEntityField *iface)
+gda_query_field_agg_get_dict_type (GdaEntityField *iface)
 {
 	g_return_val_if_fail (iface && GDA_IS_QUERY_FIELD_AGG (iface), NULL);
 	g_return_val_if_fail (GDA_QUERY_FIELD_AGG (iface)->priv, NULL);
@@ -682,15 +683,15 @@ gda_query_field_agg_save_to_xml (GdaXmlStorage *iface, GError **error)
 
 	agg = GDA_QUERY_FIELD_AGG (iface);
 
-	node = xmlNewNode (NULL, "gda_query_fagg");
+	node = xmlNewNode (NULL, (xmlChar*)"gda_query_fagg");
 	
 	str = gda_xml_storage_get_xml_id (iface);
-	xmlSetProp (node, "id", str);
+	xmlSetProp(node, (xmlChar*)"id", (xmlChar*)str);
 	g_free (str);
 
-	xmlSetProp (node, "name", gda_object_get_name (GDA_OBJECT (agg)));
+	xmlSetProp(node, (xmlChar*)"name", (xmlChar*)gda_object_get_name (GDA_OBJECT (agg)));
 	if (gda_object_get_description (GDA_OBJECT (agg)) && *gda_object_get_description (GDA_OBJECT (agg)))
-		xmlSetProp (node, "descr", gda_object_get_description (GDA_OBJECT (agg)));
+		xmlSetProp(node, (xmlChar*)"descr", (xmlChar*)gda_object_get_description (GDA_OBJECT (agg)));
 
 	obj = NULL;
 	if (gda_object_ref_activate (agg->priv->agg_ref))
@@ -699,31 +700,31 @@ gda_query_field_agg_save_to_xml (GdaXmlStorage *iface, GError **error)
 	if (obj) {
 		gchar *xmlid;
 		xmlid = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (obj));
-		xmlSetProp (node, "agg", xmlid);
+		xmlSetProp(node, (xmlChar*)"agg", (xmlChar*)xmlid);
 	}
 	else {
 		const gchar *cstr;
 
 		cstr = gda_object_ref_get_ref_name (agg->priv->agg_ref, NULL, NULL);
 		if (cstr)
-			xmlSetProp (node, "agg_name", cstr);
+			xmlSetProp(node, (xmlChar*)"agg_name", (xmlChar*)cstr);
 	}
 
 	if (! gda_query_field_is_visible (GDA_QUERY_FIELD (agg)))
-		xmlSetProp (node, "is_visible",  "f");
+		xmlSetProp(node, (xmlChar*)"is_visible",  (xmlChar*)"f");
 	if (gda_query_field_is_internal (GDA_QUERY_FIELD (agg)))
-		xmlSetProp (node, "is_internal", "t");
+		xmlSetProp(node, (xmlChar*)"is_internal", (xmlChar*)"t");
 
 	str = (gchar *) gda_query_field_get_alias (GDA_QUERY_FIELD (agg));
 	if (str && *str) 
-		xmlSetProp (node, "alias", str);
+		xmlSetProp(node, (xmlChar*)"alias", (xmlChar*)str);
 
 	/* aggregate's argument */
 	if (agg->priv->arg) {
 		xmlNodePtr argnode;
 		
-		argnode = xmlNewChild (node, NULL, "gda_query_field_ref", NULL);
-		xmlSetProp (argnode, "object", gda_object_ref_get_ref_name (agg->priv->arg, NULL, NULL));
+		argnode = xmlNewChild (node, NULL, (xmlChar*)"gda_query_field_ref", NULL);
+		xmlSetProp(argnode, (xmlChar*)"object", (xmlChar*)gda_object_ref_get_ref_name (agg->priv->arg, NULL, NULL));
 	}
 
 	return node;
@@ -741,7 +742,7 @@ gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 	g_return_val_if_fail (node, FALSE);
 
 	agg = GDA_QUERY_FIELD_AGG (iface);
-	if (strcmp (node->name, "gda_query_fagg")) {
+	if (strcmp ((gchar*)node->name, "gda_query_fagg")) {
 		g_set_error (error,
 			     GDA_QUERY_FIELD_AGG_ERROR,
 			     GDA_QUERY_FIELD_AGG_XML_LOAD_ERROR,
@@ -749,7 +750,7 @@ gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 		return FALSE;
 	}
 
-	prop = xmlGetProp (node, "id");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"id");
 	if (prop) {
 		gchar *ptr, *tok;
 		ptr = strtok_r (prop, ":", &tok);
@@ -765,26 +766,26 @@ gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 		g_free (prop);
 	}
 
-	prop = xmlGetProp (node, "name");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"name");
 	if (prop) {
 		gda_object_set_name (GDA_OBJECT (agg), prop);
 		g_free (prop);
 	}
 
-	prop = xmlGetProp (node, "descr");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"descr");
 	if (prop) {
 		gda_object_set_description (GDA_OBJECT (agg), prop);
 		g_free (prop);
 	}
 
-	prop = xmlGetProp (node, "agg");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"agg");
 	if (prop) {
 		aggref = TRUE;
 		gda_object_ref_set_ref_name (agg->priv->agg_ref, GDA_TYPE_DICT_AGGREGATE, REFERENCE_BY_XML_ID, prop);
 		g_free (prop);
 	}
 	else {
-		prop = xmlGetProp (node, "agg_name");
+		prop = (gchar*)xmlGetProp(node, (xmlChar*)"agg_name");
 		if (prop) {
 			aggref = TRUE;
 			gda_object_ref_set_ref_name (agg->priv->agg_ref, GDA_TYPE_DICT_AGGREGATE, REFERENCE_BY_NAME, prop);
@@ -792,19 +793,19 @@ gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 		}
 	}
 
-	prop = xmlGetProp (node, "is_visible");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"is_visible");
 	if (prop) {
 		gda_query_field_set_visible (GDA_QUERY_FIELD (agg), (*prop == 't') ? TRUE : FALSE);
 		g_free (prop);
 	}
 
-	prop = xmlGetProp (node, "is_internal");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"is_internal");
 	if (prop) {
 		gda_query_field_set_internal (GDA_QUERY_FIELD (agg), (*prop == 't') ? TRUE : FALSE);
 		g_free (prop);
 	}
 
-	prop = xmlGetProp (node, "alias");
+	prop = (gchar*)xmlGetProp(node, (xmlChar*)"alias");
 	if (prop) {
 		gda_query_field_set_alias (GDA_QUERY_FIELD (agg), prop);
 		g_free (prop);
@@ -816,9 +817,9 @@ gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
 		xmlNodePtr argnode = node->children;
 		dict = gda_object_get_dict (GDA_OBJECT (agg));
 		while (argnode) {
-			if (!strcmp (argnode->name, "gda_query_field_ref")) {
+			if (!strcmp ((gchar*)argnode->name, "gda_query_field_ref")) {
 				if (! agg->priv->arg) {
-					prop = xmlGetProp (argnode, "object");
+					prop = (gchar*)xmlGetProp(argnode, (xmlChar*)"object");
 					if (prop) {
 						GdaObjectRef *ref;
 						
@@ -861,7 +862,8 @@ gda_query_field_agg_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError
  */
 
 static gchar *
-gda_query_field_agg_render_as_sql (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error)
+gda_query_field_agg_render_as_sql (GdaRenderer *iface, GdaParameterList *context, GSList **out_params_used, 
+				   GdaRendererOptions options, GError **error)
 {
 	gchar *str = NULL;
 	GdaObject *base;
@@ -885,8 +887,8 @@ gda_query_field_agg_render_as_sql (GdaRenderer *iface, GdaParameterList *context
 
 			argbase = gda_object_ref_get_ref_object (agg->priv->arg);
 			if (argbase) {
-				argstr = gda_renderer_render_as_sql (GDA_RENDERER (argbase), context, 
-									  options, error);
+				argstr = gda_renderer_render_as_sql (GDA_RENDERER (argbase), context, out_params_used,
+								     options, error);
 				if (argstr) {
 					g_string_append (string, argstr);
 					g_free (argstr);
@@ -946,10 +948,9 @@ gda_query_field_agg_render_as_str (GdaRenderer *iface, GdaParameterList *context
 	agg = GDA_QUERY_FIELD_AGG (iface);
 
 	base = gda_object_ref_get_ref_object (agg->priv->agg_ref);
-
+	
 	if (base) {
 		GString *string;
-		GSList *list;
 
 		string = g_string_new (gda_object_get_name (base));
 		g_string_append (string, " (");
@@ -969,8 +970,6 @@ gda_query_field_agg_render_as_str (GdaRenderer *iface, GdaParameterList *context
 				tmpstr = gda_object_ref_get_ref_name (agg->priv->arg, NULL, NULL);
 				g_string_append (string, tmpstr);
 			}
-
-			list = g_slist_next (list);
 		}
 		g_string_append (string, ")");
 
