@@ -1,5 +1,5 @@
 /* GDA Oracle provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  * 	Tim Coleman <tim@timcoleman.com>
@@ -21,8 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
+#include <string.h>
 #include "gda-oracle.h"
 #include "gda-oracle-recordset.h"
 #include "gda-oracle-provider.h"
@@ -30,18 +31,18 @@
 #include <oci.h>
 #include <libgda/gda-data-model-private.h>
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL_BASE
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL_ROW
 
 struct _GdaOracleRecordsetPrivate {
-	GdaConnection *cnc;
+	GdaConnection           *cnc;
 	GdaOracleConnectionData *cdata;
 
-	GList *ora_values;
-	GPtrArray *rows;
-	OCIStmt *hstmt;
+	GList                   *ora_values;
+	GPtrArray               *rows;
+	OCIStmt                 *hstmt;
 
-	gint ncolumns;
-	gint nrows;
+	gint                     ncolumns;
+	gint                     nrows;
 };
 
 
@@ -57,14 +58,14 @@ static GObjectClass *parent_class = NULL;
  */
 
 static GList *
-define_columns (GdaOracleRecordsetPrivate *priv)
+define_columns (GdaOracleRecordsetPrivate *priv, GArray *col_size_array)
 {
 	GList *fields = NULL;
 	gint i;
 	gint result;
 	ub2 fake_type;
 
-	for (i = 0; i < priv->ncolumns; i += 1) {
+	for (i = 0; i < priv->ncolumns; i ++) {
 		GdaOracleValue *ora_value = g_new0 (GdaOracleValue, 1);
 
 		result = OCIParamGet (priv->hstmt,
@@ -72,8 +73,8 @@ define_columns (GdaOracleRecordsetPrivate *priv)
 				      priv->cdata->herr,
 				      (dvoid **) &(ora_value->pard),
 				      (ub4) i+1);
-		if (!gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
-					      _("Could not get the Oracle parameter")))
+		if (gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
+					     _("Could not get the Oracle parameter")))
 			return NULL;
 
 		result = OCIAttrGet ((dvoid *) (ora_value->pard),
@@ -82,8 +83,8 @@ define_columns (GdaOracleRecordsetPrivate *priv)
 				     0,
 				     OCI_ATTR_DATA_SIZE,
 				     priv->cdata->herr);
-		if (!gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
-					      _("Could not get the parameter defined size"))) {
+		if (gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
+					     _("Could not get the parameter defined size"))) {
 			OCIDescriptorFree ((dvoid *) ora_value->pard, OCI_DTYPE_PARAM);
 			return NULL;
 		}
@@ -95,8 +96,8 @@ define_columns (GdaOracleRecordsetPrivate *priv)
 				     OCI_ATTR_DATA_TYPE,
 				     priv->cdata->herr);
 		
-		if (!gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
-					      _("Could not get the parameter data type"))) {
+		if (gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
+					     _("Could not get the parameter data type"))) {
 			OCIDescriptorFree ((dvoid *) ora_value->pard, OCI_DTYPE_PARAM);
 			return NULL;
 		}
@@ -112,26 +113,48 @@ define_columns (GdaOracleRecordsetPrivate *priv)
                         break;
                 }
 
-		if (ora_value->defined_size == 0)
+		ora_value->g_type = oracle_sqltype_to_g_type (ora_value->sql_type);
+		if (col_size_array && (i < col_size_array->len)) {
+			ub2 size;
+			
+			size = g_array_index (col_size_array, ub2, i);
+			if (size > 0)
+				ora_value->defined_size = size;
+		}
+		if (ora_value->g_type == GDA_TYPE_BLOB) {
+			/* allocate a Lob locator */
+			OCILobLocator *lob;
+
+			result = OCIDescriptorAlloc ((dvoid *) priv->cdata->henv, (dvoid **) &lob, 
+						     (ub4) gda_oracle_blob_type (ora_value->sql_type), (size_t) 0, (dvoid **) 0);
+			if (gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
+						     _("Could not allocate Lob locator")))
+				return NULL;
+			ora_value->value = lob;
+			ora_value->defined_size = 0;
+		}
+		else if (ora_value->defined_size == 0)
 			ora_value->value = NULL;
-		else
-			ora_value->value = g_malloc0 (ora_value->defined_size+1);
+		else {
+			ora_value->defined_size++;
+			ora_value->value = g_malloc0 (ora_value->defined_size);
+		}
+
 		ora_value->hdef = (OCIDefine *) 0;
 		ora_value->indicator = 0;
-		ora_value->gda_type = oracle_sqltype_to_gda_type (ora_value->sql_type);
 		result = OCIDefineByPos ((OCIStmt *) priv->hstmt,
 					 (OCIDefine **) &(ora_value->hdef),
 					 (OCIError *) priv->cdata->herr,
 					 (ub4) i + 1,
 					 ora_value->value,
-					 ora_value->defined_size+1,
+					 ora_value->defined_size,
 					 (ub2) fake_type,
 					 (dvoid *) &(ora_value->indicator),
 					 (ub2 *) 0,
 					 (ub2 *) 0, 
 					 (ub4) OCI_DEFAULT);
-		if (!gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
-					      _("Could not define by position"))) {
+		if (gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
+					     _("Could not define by position"))) {
 			OCIDescriptorFree ((dvoid *) ora_value->pard, OCI_DTYPE_PARAM);
 			return NULL;
 		}
@@ -187,8 +210,8 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 			      (OCIError *) priv_data->cdata->herr,
 			      (dvoid **) &pard,
 			      (ub4) col + 1);
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
-				      _("Could not get the Oracle parameter information")))
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
+				     _("Could not get the Oracle parameter information")))
 		return;
 
 	result = OCIAttrGet ((dvoid *) pard,
@@ -197,12 +220,15 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 			     (ub4 *) & col_name_len,
 			     (ub4) OCI_ATTR_NAME,
 			     (OCIError *) priv_data->cdata->herr);
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
-				      _("Could not get the Oracle column name"))) {
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
+				     _("Could not get the Oracle column name"))) {
 		OCIDescriptorFree ((dvoid *) pard, OCI_DTYPE_PARAM);
 		return;
 	}
 	
+	if (col_name_len > ORA_NAME_BUFFER_SIZE)
+		col_name_len = ORA_NAME_BUFFER_SIZE;
+		
 	memcpy (name_buffer, pgchar_dummy, col_name_len);
 	name_buffer[col_name_len] = '\0';
 	
@@ -212,8 +238,8 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 			     (ub4 *) 0,
 			     (ub4) OCI_ATTR_DATA_TYPE,
 			     (OCIError *) priv_data->cdata->herr);
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
-				      _("Could not get the Oracle column data type"))) {
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
+				     _("Could not get the Oracle column data type"))) {
 		OCIDescriptorFree ((dvoid *) pard, OCI_DTYPE_PARAM);
 		return;
 	}
@@ -224,8 +250,8 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 			     (ub4 *) 0,
 			     (ub4) OCI_ATTR_SCALE,
 			     (OCIError *) priv_data->cdata->herr);
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
-				      _("Could not get the Oracle column scale"))) {
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
+				     _("Could not get the Oracle column scale"))) {
 		OCIDescriptorFree ((dvoid *) pard, OCI_DTYPE_PARAM);
 		return;
 	}
@@ -236,8 +262,8 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 			     (ub4 *) 0,
 			     (ub4) OCI_ATTR_IS_NULL,
 			     (OCIError *) priv_data->cdata->herr);
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
-				      _("Could not get the Oracle column nullable attribute"))) {
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
+				     _("Could not get the Oracle column nullable attribute"))) {
 		OCIDescriptorFree ((dvoid *) pard, OCI_DTYPE_PARAM);
 		return;
 	}
@@ -248,15 +274,15 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 			     0,
 			     OCI_ATTR_DATA_SIZE,
 			     priv_data->cdata->herr);
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
-				      _("Could not get the Oracle defined size"))) {
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, OCI_HTYPE_ERROR,
+				     _("Could not get the Oracle defined size"))) {
 		OCIDescriptorFree ((dvoid *) pard, OCI_DTYPE_PARAM);
 		return;
 	}
 	
 	gda_column_set_name (field_attrs, name_buffer);
 	gda_column_set_scale (field_attrs, scale);
-	gda_column_set_gdatype (field_attrs, oracle_sqltype_to_gda_type (sql_type));
+	gda_column_set_g_type (field_attrs, oracle_sqltype_to_g_type (sql_type));
 	gda_column_set_defined_size (field_attrs, defined_size);
 	
 	/* FIXME */
@@ -274,52 +300,49 @@ gda_oracle_recordset_describe_column (GdaDataModel *model, gint col)
 }
 
 static gint
-gda_oracle_recordset_get_n_rows (GdaDataModelBase *model)
+gda_oracle_recordset_get_n_rows (GdaDataModelRow *model)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
 	gint result = OCI_SUCCESS;
-	gint nrows;
+	gint nrows = -1;
 
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (model), 0);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
 	priv_data = recset->priv;
 
-	nrows = priv_data->nrows;
-
-	if (nrows >= 0) 
-		return nrows;
+	if (priv_data->nrows >= 0) 
+		return priv_data->nrows;
 
 	while (result == OCI_SUCCESS) {
 		nrows += 1;
 		result = OCIStmtFetch (priv_data->hstmt,
-					priv_data->cdata->herr,
-					(ub4) 1,
-					(ub2) OCI_FETCH_NEXT,
-					(ub4) OCI_DEFAULT);
+				       priv_data->cdata->herr,
+				       (ub4) 1,
+				       (ub2) OCI_FETCH_NEXT,
+				       (ub4) OCI_DEFAULT);
 	}
 
 	/* reset the result set to the beginning */
 	result = OCIStmtExecute (priv_data->cdata->hservice,
-				priv_data->hstmt,
-				priv_data->cdata->herr,
-				(ub4) ((OCI_STMT_SELECT == priv_data->cdata->stmt_type) ? 0 : 1),
-				(ub4) 0,
-				(CONST OCISnapshot *) NULL,
-				(OCISnapshot *) NULL,
-				OCI_DEFAULT);
+				 priv_data->hstmt,
+				 priv_data->cdata->herr,
+				 (ub4) 0,
+				 (ub4) 0,
+				 (CONST OCISnapshot *) NULL,
+				 (OCISnapshot *) NULL,
+				 OCI_DEFAULT);
 	
-	if (!gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, 
-		OCI_HTYPE_ERROR, "Could not execute Oracle statement")) {
+	if (gda_oracle_check_result (result, priv_data->cnc, priv_data->cdata, 
+				     OCI_HTYPE_ERROR, "Could not execute Oracle statement")) 
 		return 0;
-	}
 
 	return nrows;
 }
 
 static gint
-gda_oracle_recordset_get_n_columns (GdaDataModelBase *model)
+gda_oracle_recordset_get_n_columns (GdaDataModelRow *model)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (model), 0);
@@ -346,23 +369,21 @@ fetch_row (GdaOracleRecordset *recset, gint rownum)
 	row = gda_row_new (GDA_DATA_MODEL (recset), priv->ncolumns);
 
 	result = OCIStmtFetch ((OCIStmt *) priv->hstmt,
-				(OCIError *) priv->cdata->herr,
-				(ub4) 1,
-				(ub2) OCI_FETCH_NEXT,
-				(ub4) OCI_DEFAULT); 
+			       (OCIError *) priv->cdata->herr,
+			       (ub4) 1,
+			       (ub2) OCI_FETCH_NEXT,
+			       (ub4) OCI_DEFAULT); 
 	if (result == OCI_NO_DATA) 
 		return NULL;
 
-	if (!gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
-			_("Could not fetch a row"))) 
+	if (gda_oracle_check_result (result, priv->cnc, priv->cdata, OCI_HTYPE_ERROR,
+				     _("Could not fetch a row"))) 
 		return NULL;
 
-
 	i = 0;
-	for (node = g_list_first (priv->ora_values); node != NULL; 
-	     node = g_list_next (node)) {
+	for (node = priv->ora_values; node;  node = node->next) {
 		GdaOracleValue *ora_value = (GdaOracleValue *) node->data;
-		GdaValue *value = gda_row_get_value (row, i);
+		GValue *value = gda_row_get_value (row, i);
 		gda_oracle_set_value (value, ora_value, priv->cnc);
 		i += 1;
 	}
@@ -373,8 +394,8 @@ fetch_row (GdaOracleRecordset *recset, gint rownum)
 	return row;
 }
 
-static const GdaRow *
-gda_oracle_recordset_get_row (GdaDataModelBase *model, gint row)
+static GdaRow *
+gda_oracle_recordset_get_row (GdaDataModelRow *model, gint row, GError **error)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
@@ -399,7 +420,7 @@ gda_oracle_recordset_get_row (GdaDataModelBase *model, gint row)
 		return NULL;
 
 	if (row < fetched_rows) 
-		return (const GdaRow *) g_ptr_array_index (priv_data->rows, row);
+		return (GdaRow *) g_ptr_array_index (priv_data->rows, row);
 
 	gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
@@ -412,11 +433,11 @@ gda_oracle_recordset_get_row (GdaDataModelBase *model, gint row)
 
 	gda_data_model_thaw (GDA_DATA_MODEL (recset));
 
-	return (const GdaRow *) fields;
+	return fields;
 }
 
-static const GdaValue *
-gda_oracle_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
+static const GValue *
+gda_oracle_recordset_get_value_at (GdaDataModelRow *model, gint col, gint row)
 {
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 	GdaOracleRecordsetPrivate *priv_data;
@@ -430,25 +451,28 @@ gda_oracle_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
 	if (col >= priv_data->ncolumns)
 		return NULL;
 
-	fields = gda_oracle_recordset_get_row (model, row);
+	fields = gda_oracle_recordset_get_row (model, row, NULL);
 	return fields != NULL ? gda_row_get_value ((GdaRow *) fields, col) : NULL;
 }
 
 static gboolean
-gda_oracle_recordset_is_updatable (GdaDataModelBase *model)
+gda_oracle_recordset_is_updatable (GdaDataModelRow *model)
 {
 	GdaCommandType cmd_type;
 	GdaOracleRecordset *recset = (GdaOracleRecordset *) model;
 
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (recset), FALSE);
 
-	cmd_type = gda_data_model_get_command_type (GDA_DATA_MODEL (model));
+	g_object_get (G_OBJECT (model), "command_type", &cmd_type, NULL);
 
-	return cmd_type == GDA_COMMAND_TYPE_TABLE ? TRUE : FALSE;
+	return FALSE;
+	/*return cmd_type == GDA_COMMAND_TYPE_TABLE ? TRUE : FALSE;*/
 }
 
-static const GdaRow *
-gda_oracle_recordset_append_values (GdaDataModelBase *model, const GList *values)
+/* Not implemented. See bug http://bugzilla.gnome.org/show_bug.cgi?id=411811: */
+#if 0
+static GdaRow *
+gda_oracle_recordset_append_values (GdaDataModelRow *model, const GList *values, GError **error)
 {
 	GString *sql;
 	GdaRow *row;
@@ -460,7 +484,6 @@ gda_oracle_recordset_append_values (GdaDataModelBase *model, const GList *values
 	g_return_val_if_fail (GDA_IS_ORACLE_RECORDSET (recset), NULL);
 	g_return_val_if_fail (values != NULL, NULL);
 	g_return_val_if_fail (gda_data_model_is_updatable (GDA_DATA_MODEL (model)), NULL);
-	g_return_val_if_fail (gda_data_model_has_changed (GDA_DATA_MODEL (model)), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
 	priv_data = recset->priv;
@@ -496,7 +519,7 @@ gda_oracle_recordset_append_values (GdaDataModelBase *model, const GList *values
 
 	for (l = (GList *) values, i = 0; i < priv_data->ncolumns; i += 1, l = l->next) {
 		gchar *val_str;
-		const GdaValue *val = (const GdaValue *) l->data;
+		const GValue *val = (const GValue *) l->data;
 
 		if (!val) { 
 			gda_connection_add_event_string (
@@ -508,7 +531,7 @@ gda_oracle_recordset_append_values (GdaDataModelBase *model, const GList *values
 
 		if (i != 0) 
 			sql = g_string_append (sql, ", ");
-		val_str = gda_oracle_value_to_sql_string (val);
+		val_str = gda_oracle_value_to_sql_string ((GValue*) val);
 		sql = g_string_append (sql, val_str);
 
 		g_free (val_str);
@@ -523,17 +546,18 @@ gda_oracle_recordset_append_values (GdaDataModelBase *model, const GList *values
 	row = gda_row_new_from_list (GDA_DATA_MODEL (model), values);
 	g_ptr_array_add (recset->priv->rows, row);
 
-	return (const GdaRow *) row;
+	return row;
 }
+#endif
 
 static gboolean
-gda_oracle_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
+gda_oracle_recordset_remove_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	return FALSE;
 }
 
 static gboolean
-gda_oracle_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
+gda_oracle_recordset_update_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	return FALSE;
 }
@@ -554,7 +578,7 @@ static void
 gda_oracle_recordset_class_init (GdaOracleRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
+	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -564,7 +588,10 @@ gda_oracle_recordset_class_init (GdaOracleRecordsetClass *klass)
 	model_class->get_row = gda_oracle_recordset_get_row;
 	model_class->get_value_at = gda_oracle_recordset_get_value_at;
 	model_class->is_updatable = gda_oracle_recordset_is_updatable;
-	model_class->append_values = gda_oracle_recordset_append_values;
+
+        /* Not implemented. See bug http://bugzilla.gnome.org/show_bug.cgi?id=411811: */
+	model_class->append_values = /* gda_oracle_recordset_append_values */ NULL;
+
 	model_class->remove_row = gda_oracle_recordset_remove_row;
 	model_class->update_row = gda_oracle_recordset_update_row;
 }
@@ -572,7 +599,10 @@ gda_oracle_recordset_class_init (GdaOracleRecordsetClass *klass)
 static void 
 gda_oracle_free_values (GdaOracleValue *ora_value)
 {
-	g_free (ora_value->value);
+	if (ora_value->g_type == GDA_TYPE_BLOB) 
+		OCIDescriptorFree((dvoid *) ora_value->value, (ub4) gda_oracle_blob_type (ora_value->sql_type));
+	else
+		g_free (ora_value->value);
 	OCIDescriptorFree ((dvoid *) ora_value->pard, OCI_DTYPE_PARAM);
 	g_free (ora_value);
 }
@@ -599,7 +629,7 @@ gda_oracle_recordset_finalize (GObject *object)
 		GdaRow *row = (GdaRow *) g_ptr_array_index (priv_data->rows, 0);
 
 		if (row != NULL) 
-			gda_row_free (row);
+			g_object_unref (row);
 		g_ptr_array_remove_index (priv_data->rows, 0);
 	}
 	g_ptr_array_free (priv_data->rows, TRUE);
@@ -617,7 +647,7 @@ gda_oracle_recordset_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GdaOracleRecordsetClass),
 			(GBaseInitFunc) NULL,
@@ -638,7 +668,7 @@ gda_oracle_recordset_get_type (void)
 GdaOracleRecordset *
 gda_oracle_recordset_new (GdaConnection *cnc, 
 			  GdaOracleConnectionData *cdata,
-			  OCIStmt *stmthp)
+			  OCIStmt *stmthp, GArray *col_size_array)
 {
 	GdaOracleRecordset *recset;
 	ub4 parcount;
@@ -660,8 +690,8 @@ gda_oracle_recordset_new (GdaConnection *cnc,
 	recset->priv->cdata = cdata;
 	recset->priv->hstmt = stmthp;
 	recset->priv->ncolumns = parcount;
-	recset->priv->ora_values = define_columns (recset->priv);
-	recset->priv->nrows = gda_oracle_recordset_get_n_rows (GDA_DATA_MODEL_BASE (recset));
+	recset->priv->ora_values = define_columns (recset->priv, col_size_array);
+	recset->priv->nrows = gda_oracle_recordset_get_n_rows (GDA_DATA_MODEL_ROW (recset));
 
 	/* define GdaColumn attributes */
 	for (i = 0; i < recset->priv->ncolumns; i += 1) 
